const express = require("express");
const Category = require('../model/category');
const router = express.Router();
const catchAsyncErrors = require("../middleware/catchAsyncErrors");
const ErrorHandler = require("../utils/ErrorHandler");
const subCategory = require('../model/subCategory');
const Product = require("../model/product");
const shop = require("../model/shop");
const mongoose = require('mongoose');
///const users = require('../model/users');
const order = require("../model/order");
const axios = require('axios');
const product = require("../model/product");
const User=require('../model/user')

//data to get the data of total orders and orders from the last 7 days
router.get(
    "/getorders-data",
    catchAsyncErrors(async (req, res, next) => {
        try {
            const orders = await order.find();
            const lenght=orders.length;
            const sevenDaysAgo = new Date();
            console.log("sevenDaysAgo", sevenDaysAgo);
            sevenDaysAgo.setDate(sevenDaysAgo.getDate() - 7);
            console.log("the seven days ago is now", sevenDaysAgo);
            const sevendays_orders = await order.find({
                createdAt: { $gte: sevenDaysAgo }
            });
            const seven_days_ago_length=sevendays_orders.length;
            console.log("the seven days ago lenght is",seven_days_ago_length);
            res.status(200).json({
                success: true,
                lenght,
                seven_days_ago_length

            });
        } catch (error) {
            return next(new ErrorHandler(error, 400));
        }
    })
);
//api for get the total payment received and the payment received from the last 7 days
router.get(
    "/getpayment-data",
    catchAsyncErrors(async (req, res, next) => {
        try {
            const orders = await order.find();
            let  totalPayment = orders.reduce((total, order) => {
                return total + order.totalPrice;
            },0);
             const final_payment = totalPayment.toFixed(2);
             console.log("the final payment is",final_payment);
            const sevenDaysAgo = new Date();
            sevenDaysAgo.setDate(sevenDaysAgo.getDate() - 7);
            const sevendays_orders = await order.find({
                createdAt: { $gte: sevenDaysAgo }
            });
            const seven_days_ago_payment = sevendays_orders.reduce((total, order) => {
                return total + order.totalPrice;
            }, 0);
            res.status(200).json({
                success: true,
                final_payment,
                seven_days_ago_payment
            });
        } catch (error) {
            return next(new ErrorHandler(error, 400));
        }
    })
)
//api for get the total products and the products from the last 7 days
router.get(
    "/getproducts-data",
    catchAsyncErrors(async (req, res, next) => {
        try {
            // Fetch all products
            const products = await product.find();
            const length = products.length;

            // Calculate date 7 days ago
            const sevenDaysAgo = new Date();
            sevenDaysAgo.setDate(sevenDaysAgo.getDate() - 7);

            // Fetch products added in the last 7 days
            const sevendays_products = await product.find({
                createdAt: { $gte: sevenDaysAgo, $lte: new Date() }
            });

            const seven_days_ago_length = sevendays_products.length;

            console.log("Products added in the last 7 days:", sevendays_products);

            res.status(200).json({
                success: true,
                length,
                seven_days_ago_length
            });
        } catch (error) {
            return next(new ErrorHandler(error, 400));
        }
    })
);

//api to get all the users and the users which are registered from the last 7 days
router.get(
    "/getusers-data",
    catchAsyncErrors(async (req, res, next) => {
        try {
            const users = await User.find();
            const length=users.length;
            const sevenDaysAgo = new Date();
            sevenDaysAgo.setDate(sevenDaysAgo.getDate() - 7);
            const sevendays_users = await User.find({
                createdAt: { $gte: sevenDaysAgo }
            });
            const seven_days_ago_length=sevendays_users.length;
            res.status(200).json({
                success: true,
                length,
                seven_days_ago_length
            });
        } catch (error) {
            return next(new ErrorHandler(error, 400));
        }
    })
)
//api to get all the shops and the shops which are registered/created from the last 7 days
router.get(
    "/getshop-data",
    catchAsyncErrors(async (req, res, next) => {
        try {
            const shops = await shop.find();
            const length=shops.length;
            const sevenDaysAgo = new Date();
            sevenDaysAgo.setDate(sevenDaysAgo.getDate() - 7);
            const sevendays_shops = await shop.find({
                createdAt: { $lte: sevenDaysAgo }
            });
            const seven_days_ago_length=sevendays_shops.length;
            res.status(200).json({
                success: true,
                length,
                seven_days_ago_length
            });
        } catch (error) {
            return next(new ErrorHandler(error, 400));
        }
    })
)
//api to get the montly sales data with month wise data
router.get(
  "/getmonthly-data",
  catchAsyncErrors(async (req, res, next) => {
    try {
      // Aggregation pipeline to group by month and year, then calculate the total price
      const result = await order.aggregate([
        {
          $match: {
            status: "payment successful", // Filter orders with status "paymentSuccessful"
          },
        },
        {
          $project: {
            // Extract the month and year from the createdAt field
            month: { $month: "$createdAt" },
            year: { $year: "$createdAt" },
            totalPrice: 1, // Include totalPrice in the result
          },
        },
        {
          $group: {
            _id: { month: "$month", year: "$year" }, // Group by month and year
            totalPrice: { $sum: "$totalPrice" }, // Calculate totalPrice for each group
          },
        },
        {
          $project: {
            month: "$_id.month",
            year: "$_id.year",
            totalPrice: 1,
            _id: 0, // Remove _id from the final result
          },
        },
        {
          $sort: { year: 1, month: 1 }, // Sort by year and month
        },
      ]);

      // Convert numeric month to string month name
      const months = result.map((item) => {
        const monthName = new Date(item.year, item.month - 1).toLocaleString("default", { month: "long" });
        return {
          month: monthName,
          year: item.year,
          totalPrice: item.totalPrice,
        };
      });

      // Send the response with the monthly data
      res.status(200).json({
        success: true,
        months,
      });
    } catch (error) {
      return next(new ErrorHandler(error, 400));
    }
  })
);

  //api for get the daily sales data 
  router.get(
    "/getdailysales",
    catchAsyncErrors(async (req, res, next) => {
      try {
        // Aggregation pipeline to group by date and calculate total sales for each day
        const result = await order.aggregate([
          {
            $project: {
              // Extract the day, month, and year from the createdAt field
              day: { $dayOfMonth: "$createdAt" },
              month: { $month: "$createdAt" },
              year: { $year: "$createdAt" },
              totalPrice: 1, // Include totalPrice in the result
            },
          },
          {
            $group: {
              _id: { day: "$day", month: "$month", year: "$year" }, // Group by day, month, and year
              totalPrice: { $sum: "$totalPrice" }, // Sum totalPrice for each group
            },
          },
          {
            $project: {
              day: "$_id.day",
              month: "$_id.month",
              year: "$_id.year",
              totalPrice: 1,
              _id: 0, // Remove _id field from the final result
            },
          },
          {
            $sort: { year: 1, month: 1, day: 1 }, // Sort by year, month, and day
          },
        ]);
  
        // Convert the grouped data to a more readable date format
        const dailySales = result.map((item) => {
          // Format the date into a readable string (e.g., "2024-11-15")
          const dateString = new Date(item.year, item.month - 1, item.day).toLocaleDateString();
          return {
            date: dateString,
            totalPrice: item.totalPrice,
          };
        });
  
        // Send the response with the daily sales data
        res.status(200).json({
          success: true,
          dailySales,
        });
      } catch (error) {
        return next(new ErrorHandler(error, 400));
      }
    })
  );

  //api for daily average
  router.get(
    "/getdailyaverage",
    catchAsyncErrors(async (req, res) => {
      let { month, year } = req.query; 
      console.log("The month is", month, "The year is", year);
      //console.log("Parsed month:", parseInt(month), "Parsed year:", parseInt(year));
      //console.log("the length of the month is",month.length());
      console.log("the month is",month);
      month=month.toString();
      console.log("The month length is", month.length);
      if(month.length!=2){
        month="0"+month;
      }

      year=year.toString();
      console.log("the month and year afetr converting to string is",month,year);

  
      const pipeline = [
        {
          $match: {
            // shipment_status: { $ne: "Canceled" },
            status: { $in: ["payment successful"] }, 
          },
        },
        {
          $addFields: {
            convert_to_date:{
              $dateToString: {
              format: "%Y-%m-%d",
              date: "$createdAt", 
            }
             },
               convert_month:{
                 $dateToString:{
                   format:"%m",
                   date:"$createdAt",
                 }
               },
             convert_year:{
               $dateToString:{
                 format:"%Y",
                 date:"$createdAt"
               }
             }
          },
        },
        {
          $match: {
            convert_month:{$in:[month]},
            convert_year:{$in:[year]}
          },
          
        },
        {
          $group: {
            _id: 
               "$convert_to_date",
            totalOrderPrice: { $sum: "$totalPrice" }, 
          },
        },
        {
          $addFields: {
            totalOrderPrice: { $round: ["$totalOrderPrice", 2] },
          },
        },
        // {
        //   $sort: {
        //     // Sort by date in ascending order
        //     "_id.createdAt_iso": 1,
        //   },
        // },
      ];
  
      console.log("The pipeline is", pipeline);
  
      try {
        const result = await order.aggregate(pipeline);
        console.log("The result is", result);
  
        if (result && result.length > 0) {
          res.json({
            isSuccess: true,
            message: "Daily estimate amount retrieved successfully.",
            result: result,
          });
        } else {
          res.json({
            isSuccess: false,
            message: "No data found for the given month and year.",
          });
        }
      } catch (error) {
        console.error("Error while aggregating orders:", error);
        res.status(500).json({
          isSuccess: false,
          message: "Something went wrong.",
          error: error.message,
        });
      }
    })
  );

  //api for get the product details from the database like total products,approved products,rejected products
  router.get(
    "/getproduct-data",
    catchAsyncErrors(async (req, res, next) => {
        try {
            const product_data=await Product.find();
            console.log(product_data.length);
            const pipeline = [
              {
                $facet: {
                  approved_products: [
                    { $match: { status: "approved" } },
                    { $count: "count" }
                  ],
                  rejected_products: [
                    { $match: { status: "rejected" } },
                    { $count: "count" }
                  ],
                  total_products: [
                    { $count: "count" }
                  ]
                }
              },
              {
                $project: {
                  approvedproduct: { $arrayElemAt: ["$approved_products.count", 0] },
                  rejectedproduct: { $arrayElemAt: ["$rejected_products.count", 0] },
                  totalproduct: { $arrayElemAt: ["$total_products.count", 0] }
                }
              }
            ];
            
            const result=await product.aggregate(pipeline);
            console.log("the result after the aggregation is",result);


            res.status(200).json({
                success: true,
                result
                
            });
        } catch (error) {
            return next(new ErrorHandler(error, 400));
        }
    })
)
  
  
  


module.exports = router;
