const express = require("express");
const Category=require('../model/category');
const router = express.Router();
const catchAsyncErrors = require("../middleware/catchAsyncErrors");
const ErrorHandler = require("../utils/ErrorHandler");
const subCategory=require('../model/subCategory');
const Product = require("../model/product");
const mongoose=require('mongoose');
const { upload } = require("../multer");

//api for create a new category
router.post(
    "/create-category",upload.single('image'),
    catchAsyncErrors(async (req, res, next) => {
      try {
        const image = req.file;
        req.body.image = image.filename;         console.log(req.body.image);
        const category = await Category.create(req.body);
        res.status(201).json({
          success: true,
          category,
        });
      } catch (error) {
        return next(new ErrorHandler(error, 400));
      }
    })
  );
//api for  get all category
router.get(
    "/getallcategory",
    catchAsyncErrors(async (req, res, next) => {
      try {
        // Use `distinct` to get unique category names
        const categories = await Category.find();
        res.status(200).json({
          success: true,
          categories,
        });
      } catch (error) {
        return next(new ErrorHandler(error, 400));
      }
    })
  );
  //api for delete any category using id
  router.delete(
    "/deletecategory/:id",
    catchAsyncErrors(async(req,res,next)=>{
        try{
            await Category.findByIdAndDelete(req.params.id)
            res.status(200).json({
                success:true,
            })
        }catch(error){
            return next(new ErrorHandler(error,400))
        }
    })
  )
  //api for create subCategory
  router.post(
    "/create-subcategory",
    catchAsyncErrors(async (req, res, next) => {
      try {
        console.log("the data coming from the frontend is", req.body);
        const subcategory = await subCategory.create(req);
        res.status(201).json({
          success: true,
          subcategory,
        });
      } catch (error) {
        return next(new ErrorHandler(error, 400));
      }
    })
  );

  //api for find the subCategories by Category Id
  router.get(
    "/getsubcategory/:categoryId",
    catchAsyncErrors(async (req, res, next) => {
      try {
        const {categoryId}=req.params;
        const isCorrectCategoryId=await Category.findById({_id:categoryId});
        if(!isCorrectCategoryId){
          return res.status(404).json({success:false,message:"This is not a valid categoryId"})
        }
       const subCategories=await subCategory.find({parentCategory:categoryId});
        res.status(200).json({
          success: true,
          subCategories,
        });
      } catch (error) {
        return next(new ErrorHandler(error, 400));
      }
    })
  );
// //api to find the products by subCategory Id
//   router.get(
//     "/getproductBysubcategory/:subcategoryId",
//     catchAsyncErrors(async (req, res, next) => {
//       try {
//         const { subcategoryId } = req.params;
  
//         const subcategoryWithProducts = await subCategory.aggregate([
//           {
//             $match: { _id: new mongoose.Types.ObjectId(subcategoryId) }
//           },
//           {
//             $lookup: {
//               from: "products", 
//               localField: "products",
//               foreignField: "_id",
//               as: "productDetails"
//             }
//           },
//           {
//             $project: {
//               _id: 1,
//               name: 1,
//               productDetails: 1
//             }
//           }
//         ]);
  
//         if (!subcategoryWithProducts || subcategoryWithProducts.length === 0) {
//           return res.status(404).json({ success: false, message: "No products found for this subcategory" });
//         }
  
//         res.status(200).json({
//           success: true,
//           products: subcategoryWithProducts[0].productDetails,
//         });
//       } catch (error) {
//         return next(new ErrorHandler(error, 400));
//       }
//     })
//   );
  
  //api for fetch the products by the subCategoryId
  router.get(
    "/getproductsBySubId/:subcategoryId",
    catchAsyncErrors(async (req, res, next) => {
      try {
        const {subcategoryId}=req.params;
         const validsubcategory=await subCategory.findById({_id:subcategoryId});
        if(!validsubcategory){
          return res.status(404).json({success:false,message:"subcategory not found"})
        }
          const products = await Product.find({
            subcategoryId:new  mongoose.Types.ObjectId(subcategoryId) , 
            status: "approved",
            draft_status:"no",          
            stock:{$gt:0}          
          });
          console.log("Gettung profiys",products)
          const response={
            success:true,
            products:products
          }
          console.log("response data send is ",response)
        res.status(200).json(response);
      } catch (error) {
        return next(new ErrorHandler(error, 400));
      }
    })
  );
  
  module.exports = router;


