const express = require("express");
const Item=require('../model/item');
const Category=require('../model/category')
const router = express.Router();
const catchAsyncErrors = require("../middleware/catchAsyncErrors");

//api for create item
router.post(
    "/create-item",
    catchAsyncErrors(async (req, res, next) => {
      try {
        const categoryId=req.body.categoryId;
        const isPresent=await Category.findById({_id:categoryId})
        if(!isPresent){
            return res.status(400).json({message:"This is not a valid category"})
        }
        const category = await Item.create(req.body);
        res.status(201).json({
          success: true,
          category,
        });
      } catch (error) {
        return next(new ErrorHandler(error, 400));
      }
    })
  );
  //api for get all the item
  router.get(
    "/getallitem",
    catchAsyncErrors(async (req, res, next) => {
      try {
        const item = await Item.find();
        res.status(200).json({
          success: true,
          item,
        });
      } catch (error) {
        return next(new ErrorHandler(error, 400));
      }
    })
  );

  //api for delete a item by Id
  router.delete(
    "/deleteitem/:id",
    catchAsyncErrors(async(req,res,next)=>{
        try{
            await Item.findByIdAndDelete(req.params.id)
            res.status(200).json({
                success:true,
            })
        }catch(error){
            return next(new ErrorHandler(error,400))
        }
    })
  )
  
  //api for find items by categoryId
  router.get(
    "/finditembycategory/:id",
    catchAsyncErrors(async (req, res, next) => {
        try{
         const items=await Item.find({
            categoryId:req.params.id
         })
         res.status(200).json({
            success:true,
            items
        })

        }catch(error){
            return next(new ErrorHandler(error,400))
        }
 })
)

  module.exports = router;
