const express = require("express");
const { isSeller, isAuthenticated, isAdmin } = require("../middleware/auth");
const catchAsyncErrors = require("../middleware/catchAsyncErrors");
const router = express.Router();
const Product = require("../model/product");
const Order = require("../model/order");
const Shop = require("../model/shop");
const { upload } = require("../multer");
const ErrorHandler = require("../utils/ErrorHandler");
const fs = require("fs");
const subCategory=require('../model/subCategory');
const axios = require('axios');
const order = require("../model/order");
const generateShipRocketToken =require('../utils/generateShipRocketToken');
const SibApiV3Sdk=require('sib-api-v3-sdk');
const User=require('../model/user');





// create product
router.post(
  "/create-product",
  upload.array("images"),
  catchAsyncErrors(async (req, res, next) => {
    try {
      const subcategory=await subCategory.findById({_id:req.body.subcategoryId});
      if(!subcategory){
        return next(new ErrorHandler("subcategory not found",404))
      }
      console.log(req.body)
      
      const shopId = req.body.shopId;
      console
      const shop = await Shop.findById(shopId);
      if (!shop) {
        return next(new ErrorHandler("Shop Id is invalid!", 400));
      } else {
        const files = req.files;
        const imageUrls = files.map((file) => `${file.filename}`);
        console.log("the images are",imageUrls);

        const productData = req.body;
        productData.images = imageUrls;
        productData.shop = shop;

        const product = await Product.create(productData);
      console.log("the product is",product);
      

      var address=product.addresses.address1+","+ "10";
      if(address.length!=10){
         address="      "+address
      }
      const address_2=product.addresses.address2 || "";
      const city=product.addresses.city;
      const pin_code=product.addresses.zipCode;
      const state=product.addresses.state;
      const country=product.addresses.country;
      const name=product.shop.name;
      var name2=product.shop.name;
      if(name2.length>8){
        name2=name2.substring(0,8)
      }
      console.log("the name afetr substring is",name);
      
      const email=product.shop.email;
      const phone=product.shop.phoneNumber;
      var pickup_location=name2+product._id;
      console.log("the adress and the seller details are",address,address_2,city,pin_code,state,country,name,email,phone,pickup_location);

      const params2 = {
        pickup_location,
        name,
        email,
        phone,
        address,
        address_2,
        city,
        state,
        country,
        pin_code
    };
    const tokendata=await generateShipRocketToken();
    console.log("the generated token is",tokendata)

      const create_new_pickuplocation_url = `https://apiv2.shiprocket.in/v1/external/settings/company/addpickup`;
        const response = await axios.post(create_new_pickuplocation_url,params2,
    {
     
      headers: {
          'Content-Type': 'application/json',
          'Authorization': `Bearer ${tokendata}`
      } 
  });
  const pickup_location_data=response.data.address.pickup_code;
  console.log(response.data);
  console.log("the pickup location data is",pickup_location_data);
   const order = await Product.findById({_id:product._id});
   console.log("the order is",order);
 if (!order) {
  throw new Error('Order not found');
 }

 order.pickup_location = pickup_location_data;
await order.save();
  // order.pickup_location=pickup_location_data;
  // await order.save();
  res.status(201).json({
    success: true,
    product,
  });

      }
    } catch (error) {
      return next(new ErrorHandler(error, 400));
    }
  })
);
//api for get all category
router.get(
  "/getallcategory",
  catchAsyncErrors(async (req, res, next) => {
    try {
      // Use `distinct` to get unique category names
      const categories = await Product.distinct('category');
      
      res.status(201).json({
        success: true,
        categories,
      });
    } catch (error) {
      return next(new ErrorHandler(error, 400));
    }
  })
);
//api for send email to the product owner when someone show interested in their product
router.post(
  "/interested/:product_id",
  catchAsyncErrors(async (req, res, next) => {
    try{
      const{product_id}=req.params;
      console.log("the params id is",product_id);
      const product_data=await Product.findById({_id:product_id});
      console.log("the product data is",product_data);
      const product_owner_email=product_data.shop.email;
      const product_name=product_data.name;
      const product_ownername=product_data.shop.name;
      await sendInterestEmail(product_owner_email,product_name,product_ownername);
      async function sendInterestEmail( product_owner_email, product_name,product_ownername) {
        var defaultClient = SibApiV3Sdk.ApiClient.instance;
        // Configure API key authorization: api-key
        var apiKey = defaultClient.authentications['api-key'];
        console.log("the api key is", apiKey)
        apiKey.apiKey = process.env.MAILAPIKEY;
        var apiInstance = new SibApiV3Sdk.TransactionalEmailsApi();
        var adminMail = new SibApiV3Sdk.SendSmtpEmail();
        const getIndianTime = () => {
            return new Date().toLocaleTimeString('en-IN', { 
                timeZone: 'Asia/Kolkata', 
                hour: '2-digit', 
                minute: '2-digit', 
                second: '2-digit', 
                hour12: false 
            });
        };
      
        try {
      
            adminMail = {
                to: [
                    
                       { email: product_owner_email},
                       { email: 'gyanaranjan@ajatus.co.in'}
                ],
                from: {
                    email: 'support@ceibagreen.com'
                },
                templateId: 38,
                params: {
                   
                    product_name: product_name,
                    product_ownername:product_ownername
                   
                },
                subject: `Someone has shown interest in your product ${product_name}`,
                headers: {
                    'X-Mailin-custom': 'custom_header_1:custom_value_1|custom_header_2:custom_value_2'
                },
            }
            const data = await apiInstance.sendTransacEmail(adminMail);
            console.log("mail send successfully", data.messageId);
            //console.log(productName)
        } catch (error) {
            console.log(error);
        }
      }

      res.status(201).json({
        success: true,
      });
    } catch (error) {
      return next(new ErrorHandler(error, 400));
    }
  })
);
// get all products of a shop
router.get(
  "/get-all-products-shop/:id",
  catchAsyncErrors(async (req, res, next) => {
    try {
      const products = await Product.find({ shopId: req.params.id });

      res.status(201).json({
        success: true,
        products,
      });
    } catch (error) {
      return next(new ErrorHandler(error, 400));
    }
  })
);
//api for fetch the draft products by the userId
router.get(
  "/getdraft-items/:id",
  catchAsyncErrors(async (req, res, next) => {
    try {
      const {id}=req.params;
      console.log("the path parameter is",id);
      
      //first fetch the product by the shopId/userId
      const products = await Product.find({ shopId: id ,draft_status:"yes"});
      console.log("the products is",products);

      res.status(201).json({
        success: true,
        products,
      });
    } catch (error) {
      return next(new ErrorHandler(error, 400));
    }
  })
);

//api for fetch the product according to the productId
router.get(
  "/getproduct-byId/:product_id",
  catchAsyncErrors(async (req, res, next) => {
    try {
      const { product_id } = req.params;
      const products = await Product.find({ _id: product_id });

      res.status(201).json({
        success: true,
        products,
      });
    } catch (error) {
      return next(new ErrorHandler(error, 400));
    }
  })
);
//api for search using query parameter
router.get(
  "/search-product",
  catchAsyncErrors(async (req, res, next) => {
    try {
      // Fetch the query coming from the query params
      const { query } = req.query;

      // Ensure the query parameter is provided
      if (!query) {
        return res.status(400).json({
          success: false,
          message: "Please enter the product name to search",
        });
      }

      // Find products matching the search query and with status other than 'approved' or 'rejected'
      const products = await Product.find({
        $or: [
          { name: { $regex: query, $options: "i" } },
          { description: { $regex: query, $options: "i" } },
        ],
        status: { $nin: ["approved", "rejected"] },
      });

      // Return the filtered products
      res.status(201).json({
        success: true,
        products,
      });
    } catch (error) {
      return next(new ErrorHandler(error, 400));
    }
  })
);



// delete product of a shop
router.delete(
  "/delete-product-byId/:product_id",
  catchAsyncErrors(async (req, res, next) => {
    try {
      const {product_id} = req.params;
      console.log("the params is",req.params);
      console.log("the product id is",product_id);

      const productData = await Product.findById(product_id);
      console.log("the product data is",productData);

      productData.images.forEach((imageUrl) => {
        const filename = imageUrl;
        const filePath = `uploads/${filename}`;

        fs.unlink(filePath, (err) => {
          if (err) {
            console.log(err);
          }
        });
      });

      const product = await Product.findOneAndDelete({_id:product_id});
      console.log("the product is after delete the images is",product);

      if (!product) {
        return next(new ErrorHandler("Product not found with this id!", 500));
      }

      res.status(201).json({
        success: true,
        message: "Product Deleted successfully!",
        product
      });
    } catch (error) {
      return next(new ErrorHandler(error, 400));
    }
  })
);


// get all products who are verified by the admin
router.get(
  "/get-all-products",
  catchAsyncErrors(async (req, res, next) => {
    try {
      const products = await Product.find({
        $and: [
          { status: { $in: ["verified", "approved"] } }, 
          { stock: { $gt: 0 } }, 
        ],
      }).sort({ createdAt: -1 });

      console.log("The product is", products);

      res.status(200).json({
        success: true,
        products,
      });
    } catch (error) {
      return next(new ErrorHandler(error.message, 400));
    }
  })
);


//get all the productsto show in the admin web ui
router.get(
  "/getallproducts",
  catchAsyncErrors(async (req, res, next) => {
    try {
      const products = await Product.find({ status: { $nin: ["approved", "rejected"] },draft_status: { $ne: "yes" }, })
        .sort({ createdAt: -1 });

      //console.log("the product is",products);

      res.status(201).json({
        success: true,
        products,
      });
    } catch (error) {
      return next(new ErrorHandler(error, 400));
    }
  })
);
//api for find all the products by the subcategoryId
router.get(
  "/getallproducts-by-subcategoryId/:subcategoryId",
  catchAsyncErrors(async (req, res, next) => {
    try {
     const subcategoryId = req.params.subcategoryId;
     const products= await Product.find({ subcategoryId: subcategoryId });
     //console.log("the product is",products);

      //console.log("the product is",products);

      res.status(201).json({
        success: true,
        products,
      });
    } catch (error) {
      return next(new ErrorHandler(error, 400));
    }
  })
);

//api for accept the status from the admin side and modify the status according to that
router.post(
  "/getstatus-from-admin-approve/:product_id",
  catchAsyncErrors(async (req, res, next) => {
    try {
      
      const product_id= req.params.product_id;
      console.log("the product is",product_id);
      const product=await Product.findById({_id:product_id});
      console.log("the product is",product);
      const productName=product.name;
      const sellername=product.shop.name;
      const sellerEmail=product.shop.email;
       console.log("the product status is",product.status);
        product.status='approved'
        await product.save();
      console.log(product);
      //call the send email function here to send the email that his product has been approved
       await sendApprovedEmail(productName,sellerEmail,sellername);
       res.status(200).json({message:"product approved"})

      //logic for fire the email after the product gets approved from the admin side
      async function sendApprovedEmail( productName, sellerEmail,sellername) {
        var defaultClient = SibApiV3Sdk.ApiClient.instance;
        // Configure API key authorization: api-key
        var apiKey = defaultClient.authentications['api-key'];
        console.log("the api key is", apiKey)
        apiKey.apiKey = process.env.MAILAPIKEY;
        var apiInstance = new SibApiV3Sdk.TransactionalEmailsApi();
        var adminMail = new SibApiV3Sdk.SendSmtpEmail();
        const getIndianTime = () => {
            return new Date().toLocaleTimeString('en-IN', { 
                timeZone: 'Asia/Kolkata', 
                hour: '2-digit', 
                minute: '2-digit', 
                second: '2-digit', 
                hour12: false 
            });
        };
      
        try {
      
            adminMail = {
                to: [
                       { email: sellerEmail},
                       { email: 'gyanaranjan@ajatus.co.in'}  
                ],
                from: {
                    email: 'support@ceibagreen.com'
                },
                templateId: 36,
                params: {
                    product_name: productName,
                    customer_name: sellername,
                },
                subject: `Congrats Your Product ${productName} Approved by Admin..`,
                headers: {
                    'X-Mailin-custom': 'custom_header_1:custom_value_1|custom_header_2:custom_value_2'
                },
            }
            const data = await apiInstance.sendTransacEmail(adminMail);
            console.log("Product approval mail send successfully", data.messageId);
            console.log(productName)
        } catch (error) {
            console.log(error);
        }
      }
      
    } catch (error) {
      return next(new ErrorHandler(error, 400));
    }
  })
);
//if admin press rejectd from admin side then this api will be fired
router.post(
  "/getstatus-from-admin-rejected/:product_id",
  catchAsyncErrors(async (req, res, next) => {
    try {
      const { product_id } = req.params;
      const { reason } = req.body; // Destructure rejection reason from request body
      console.log("the product id is", product_id);
      console.log("the rejection reason is", reason);

      if (!reason || !reason.trim()) {
        return res.status(400).json({ message: "Rejection reason is required" });
      }

      const product = await Product.findById(product_id);
      const productName=product.name;
      const sellername=product.shop.name;
      const sellerEmail=product.shop.email;

      console.log("the product seller details is",productName,sellername,sellerEmail);

      if (!product) {
        return res.status(404).json({ message: "Product not found" });
      }

      product.status = "rejected";
      product.reason_of_rejection = reason;
      await product.save();

      
      //call the function here to fire the email that the product has been rejected
      await sendRejectionEmail(productName,sellerEmail,sellername,reason);
      res.status(200).json({ message: "Product rejected successfully", product });

      //logic for firing the email after the product rejected from the admin
      async function sendRejectionEmail( productName, sellerEmail,sellername,reason) {
        var defaultClient = SibApiV3Sdk.ApiClient.instance;
        // Configure API key authorization: api-key
        var apiKey = defaultClient.authentications['api-key'];
        console.log("the api key is", apiKey)
        apiKey.apiKey = process.env.MAILAPIKEY;
        var apiInstance = new SibApiV3Sdk.TransactionalEmailsApi();
        var adminMail = new SibApiV3Sdk.SendSmtpEmail();
        const getIndianTime = () => {
            return new Date().toLocaleTimeString('en-IN', { 
                timeZone: 'Asia/Kolkata', 
                hour: '2-digit', 
                minute: '2-digit', 
                second: '2-digit', 
                hour12: false 
            });
        };
      
        try {
      
            adminMail = {
                to: [
                       { email: sellerEmail},
                       { email: 'gyanaranjan@ajatus.co.in'}  
                ],
                from: {
                    email: 'support@ceibagreen.com'
                },
                templateId: 37,
                params: {
                    product_name: productName,
                    customer_name: sellername,
                    rejection_reason: reason,
                },
                subject: `Important: Your Product Submission Has Been Rejected`,
                headers: {
                    'X-Mailin-custom': 'custom_header_1:custom_value_1|custom_header_2:custom_value_2'
                },
            }
            const data = await apiInstance.sendTransacEmail(adminMail);
            console.log("Product approval mail send successfully", data.messageId);
            console.log(productName)
        } catch (error) {
            console.log(error);
        }
      }
    } catch (error) {
      return next(new ErrorHandler(error.message || "Internal server error", 500));
    }
  })
);
//api for change the status the product which is in draft if he wants to post
router.put(
  "/edit-draft-status/:productId",
  catchAsyncErrors(async(req,res,next)=>{
    try{
    //fetch the productId from the params
    const{productId}=req.params;
    const products=await Product.findById(productId);
    //console.log("the product is",products);
    console.log(products.draft_status);
    if(products.draft_status=="yes"){
      products.draft_status="no";
      await products.save();
    }
    res.status(200).json({message:"product draft status updated successfully"})
  }catch(error){
    return next(new ErrorHandler(error.message || "Internal server error to change the draft status", 500));
  }

  }
)
)
//api for edit the product details from the admin side
router.put(
  "/edit-products-details/:product_id",
  catchAsyncErrors(async(req,res,next)=>{
    try{
    const{product_id}=req.params;
    const productData=await Product.findById(product_id);
    productData.name=req.body.name;
    productData.description=req.body.description;
    //productData.price=product.price;
    productData.fabric=req.body.fabric;
    productData.size=req.body.size;
    await productData.save();
    res.status(200).json({message:"product details updated successfully"})
    
  }catch(error){
    return next(new ErrorHandler(error.message || "Internal server error to change the draft status", 500));
  }

  }
)
)
//api for get the product data of the user like total number of products listed,no of rejected products and no of approved products
router.get(
  "/get-product-data/:userId",
  catchAsyncErrors(async (req, res, next) => {
    try {
      
      const userData=await User.findById(req.params.userId);
      const phoneNumber=userData.phoneNumber;
      const productData=await Product.find({'shop.phoneNumber':phoneNumber});
      //console.log("the product listed by the user is",productData);

      // Count total orders
      const totalproducts = productData.length;

      // Count where the product status is "approved"
      const approvedProducts = productData.filter(
        (product) => product.status === "approved"||product.status === "Approved"
      ).length;

      // Count orders where product status is "rejected"
      const rejectedProducts = productData.filter(
        (product) => product.status === "rejected"||product.status === "rejected"
      ).length;
      //count the products whose status is still pending now not approved or rejected
      const pendingProducts=productData.filter((product) => product.status === "draft"||product.status === "draft").length
      

      // Respond with the counts
      res.status(200).json({
        success: true,
        totalproducts,
        approvedProducts,
        rejectedProducts,
        pendingProducts
      });
    } catch (error) {
      return next(new ErrorHandler(error.message, 500));
    }
  })
);


// review for a product
router.put(
  "/create-new-review",
  isAuthenticated,
  catchAsyncErrors(async (req, res, next) => {
    try {
      const { user, rating, comment, productId, orderId } = req.body;

      const product = await Product.findById(productId);

      const review = {
        user,
        rating,
        comment,
        productId,
      };

      const isReviewed = product.reviews.find(
        (rev) => rev.user._id === req.user._id
      );

      if (isReviewed) {
        product.reviews.forEach((rev) => {
          if (rev.user._id === req.user._id) {
            (rev.rating = rating), (rev.comment = comment), (rev.user = user);
          }
        });
      } else {
        product.reviews.push(review);
      }

      let avg = 0;

      product.reviews.forEach((rev) => {
        avg += rev.rating;
      });

      product.ratings = avg / product.reviews.length;

      await product.save({ validateBeforeSave: false });

      await Order.findByIdAndUpdate(
        orderId,
        { $set: { "cart.$[elem].isReviewed": true } },
        { arrayFilters: [{ "elem._id": productId }], new: true }
      );

      res.status(200).json({
        success: true,
        message: "Reviwed succesfully!",
      });
    } catch (error) {
      return next(new ErrorHandler(error, 400));
    }
  })
);

// all products --- for admin
router.get(
  "/admin-all-products",
  isAuthenticated,
  isAdmin("Admin"),
  catchAsyncErrors(async (req, res, next) => {
    try {
      const products = await Product.find().sort({
        createdAt: -1,
      });
      res.status(201).json({
        success: true,
        products,
      });
    } catch (error) {
      return next(new ErrorHandler(error.message, 500));
    }
  })
);

module.exports = router;
