const express = require("express");
const ErrorHandler = require("./middleware/error");
const connectDatabase = require("./db/Database");
const app = express();
const order = require("./model/order");
const cron = require('node-cron');
const moment = require('moment');
const Order = require("./model/order");
const axios = require('axios');
const product = require("./model/product");
const SibApiV3Sdk=require('sib-api-v3-sdk');
const generateShipRocketToken =require('./utils/generateShipRocketToken');








const cookieParser = require("cookie-parser");
const bodyParser = require("body-parser");
const cors = require("cors");
const path = require("path");

// config
if (process.env.NODE_ENV !== "PRODUCTION") {
  require("dotenv").config({
    path: "config/.env",
  });
}
// connect db
connectDatabase();

// create server
const server = app.listen(process.env.PORT, () => {
  console.log(`Server is running on http://localhost:${process.env.PORT}`);
});

// middlewares
app.use(express.json());
app.use(cookieParser());
// Enable CORS for all routes

app.use(
  cors()
);

app.use("/", express.static("uploads"));

app.get("/test", (req, res) => {
  res.send("Hello World!");
});

app.use(bodyParser.urlencoded({ extended: true}));

// why bodyparser?
// bodyparser is used to parse the data from the body of the request to the server (POST, PUT, DELETE, etc.)

// config
if (process.env.NODE_ENV !== "PRODUCTION") {
  require("dotenv").config({
    path: "config/.env",
  });
}

app.get("/", (req, res) => {
  res.send("Hello World!");
});

// routes
const user = require("./controller/user");
const shop = require("./controller/shop");
const productt = require("./controller/product");
const event = require("./controller/event");
const coupon = require("./controller/coupounCode");
const payment = require("./controller/payment");
const orderr = require("./controller/order");
const message = require("./controller/message");
const conversation = require("./controller/conversation");
const withdraw = require("./controller/withdraw");
const category=require("./controller/category");
const Item=require('./controller/item');
const shiprocket=require('./controller/shipRocket')
const dashboard=require('./controller/DashBoard');
app.use("/api/v2/withdraw", withdraw);

// end points
app.use("/api/v2/user", user);
app.use("/api/v2/conversation", conversation);
app.use("/api/v2/message", message);
app.use("/api/v2/order", orderr);
app.use("/api/v2/shop", shop);
app.use("/api/v2/product", productt);
app.use("/api/v2/event", event);
app.use("/api/v2/coupon", coupon);
app.use("/api/v2/payment", payment);
app.use("/api/v2/category",category);
app.use("/api/v2/item", Item);
app.use("/api/v2/shiprocket",shiprocket);
app.use("/api/v2/dashboard",dashboard);

cron.schedule('*/2 * * * *', async () => {
  console.log("Running cron to generate the AWB code and label");
  //to generate a new pickup location from the address of product
  try {
     
    const oneHourAgo = moment().subtract(1, 'minutes').toISOString();
    const pendingOrders = await Order.find({
      createdAt: { $lte: oneHourAgo },
      awb_code: { $exists: false },
      lebel:{$exists:false},
      status: { $in: ["payment successful"] },
    });
    console.log(pendingOrders);
    const shipment_id=pendingOrders[0].shipment_id;
    if(!shipment_id){
      console.log('shipment id not found there is no order now')
    }
    //logic to fetch the ordcer details from order table using shipment_id
    const orderDetails = await Order.find({ shipment_id: shipment_id });
    console.log("the order details is",orderDetails);

    const productId=orderDetails[0].cart[0].productId;
    console.log("**********",orderDetails[0].cart[0].productId);
    console.log("the product id isss",productId);
    //now fetch the product details from the productId
    const productDetails = await product.find({ _id: productId });
    console.log("the product details is",productDetails);
    const productName=productDetails[0].name;
    const sellerEmail=productDetails[0].shop.email;
    const sellerName=productDetails[0].shop.name;
    console.log("the seller details in the server.js page is",sellerName,sellerEmail);
    //this is the logic to fetch the courier_id from the cart when the user place order
    const courier_id=orderDetails[0].cart[0].courier_id;
    
    //generate the token here and use with the api
    const token=await generateShipRocketToken();

    for (const order of pendingOrders) {
      console.log("the shipment id of the pending orders is",pendingOrders[0].shipment_id);
     // api for generate awb number
      const response3 = await axios.post('https://apiv2.shiprocket.in/v1/external/courier/assign/awb', {
        shipment_id:pendingOrders[0].shipment_id,
        //courier_id:courier_id
      }, {
        headers: {
          Authorization: `Bearer ${token}`
        }
      });
      if (response3.data) {
        // Update order with the new AWB code
        order.awb_code = response3.data.response.data.awb_code;
        order.courier_order_id=response3.data.response.data.order_id;
        await order.save();

        // console.log(`AWB generated for order ${order._id}: ${response.data.awb_code}`);
      }
      console.log(response3.data);
      console.log("the awb code for saving in database is",response3.data.response.data.awb_code);
      //api for generate the label/invoice for the product
      const response4 = await axios.post('https://apiv2.shiprocket.in/v1/external/courier/generate/label', {
        shipment_id:[pendingOrders[0].shipment_id],
      }, {
        headers: {
          Authorization: `Bearer ${token}`        
        }
      });
      console.log(response4.data);
      const lebel=response4.data.label_url;
      if(response4.data){
        
        order.lebel=response4.data.label_url;
        await order.save();
      }

     // api for create a pickup from the seller end
      const response5 = await axios.post('https://apiv2.shiprocket.in/v1/external/courier/generate/pickup', {
        shipment_id:[pendingOrders[0].shipment_id],
      }, {
        headers: {
          Authorization: `Bearer ${token}`       
         }
      });
      if(response5.data){
        console.log("the pickup date is",response5.data.response.pickup_scheduled_date)
        const pickup_date = response5.data.response.pickup_scheduled_date; // Example: "2024-11-14T15:45:00Z"
         const dateObj = new Date(pickup_date);

        // Format to dd-mm-yy
        const formattedDate = dateObj.toLocaleDateString('en-GB', {
      year: '2-digit',
      month: '2-digit',
      day: '2-digit',
});

console.log(formattedDate);
        await sendEmailToseller(productName,sellerEmail,sellerName,formattedDate);
      }
      async function sendEmailToseller( productName, sellerEmail,sellername,pickup_date) {
        var defaultClient = SibApiV3Sdk.ApiClient.instance;
        // Configure API key authorization: api-key
        var apiKey = defaultClient.authentications['api-key'];
        console.log("the api key is", apiKey)
        apiKey.apiKey = process.env.MAILAPIKEY;
        // console.log("the api key after this is", apiKey)
        // Uncomment below two lines to configure authorization using: partner-key
        // var partnerKey = defaultClient.authentications['partner-key'];
        // partnerKey.apiKey = 'YOUR API KEY';
        var apiInstance = new SibApiV3Sdk.TransactionalEmailsApi();
        var adminMail = new SibApiV3Sdk.SendSmtpEmail();
        const getIndianTime = () => {
            return new Date().toLocaleTimeString('en-IN', { 
                timeZone: 'Asia/Kolkata', 
                hour: '2-digit', 
                minute: '2-digit', 
                second: '2-digit', 
                hour12: false 
            });
        };
      
        try {
      
            adminMail = {
                to: [
                    
                       { email: sellerEmail},
                       { email: 'gyanaranjan@ajatus.co.in'}
                      //  { email: 'omprakash.sadangi@ajatus.co.in'},
                          
                ],
                from: {
                    email: 'support@ceibagreen.com'
                },
                templateId: 32,
                params: {
                    // parameter: 'recycle_pay',
                    // subject: `Item updated ${itemName}`,
                    // body: `Item updated ${itemName} from ${previousPrice} to ${updatedPrice}`,
                    // date: new Date().toLocaleDateString(),
                    // time: getIndianTime(),
                    product_name: productName,
                    customer_name: sellername,
                    pickup_date:pickup_date,
                    label_url:lebel
                },
                subject: `Product Pickup Details and Instruction`,
                headers: {
                    'X-Mailin-custom': 'custom_header_1:custom_value_1|custom_header_2:custom_value_2'
                },
            }
            const data = await apiInstance.sendTransacEmail(adminMail);
            console.log("mail send successfully", data.messageId);
            console.log(productName)
        } catch (error) {
            console.log(error);
        }
      }
    }
    
  } catch (error) {
    console.log("Error generating AWB:",error);
  }
});



cron.schedule('*/25 * * * *', async () => {
  console.log("Running cron to check the shipment status and send email to the shipped items");
  try {
    const token = await generateShipRocketToken();

    async function productShippedEmail(productName, BuyerEmail, Buyername,  awbnumber, delivery_address,zip_code,city,state) {
      var defaultClient = SibApiV3Sdk.ApiClient.instance;
      var apiKey = defaultClient.authentications['api-key'];
      apiKey.apiKey = process.env.MAILAPIKEY;

      var apiInstance = new SibApiV3Sdk.TransactionalEmailsApi();
      var adminMail = new SibApiV3Sdk.SendSmtpEmail();

      try {
        adminMail = {
          to: [
            { email: BuyerEmail },
            { email: 'gyanaranjan@ajatus.co.in' }
          ],
          from: {
            email: 'support@ceibagreen.com'
          },
          templateId: 34,
          params: {
            product_name: productName,
            Buyername: Buyername,
            awbnumber: awbnumber,
            delivery_address: delivery_address,
            zip_code: zip_code,
            city:city,
            state:state
          },
          subject: `Product Shipped..Order is on the way`,
        };

        const data = await apiInstance.sendTransacEmail(adminMail);
        console.log("Mail sent successfully:", data.messageId);
      } catch (error) {
        console.log("Error sending email:", error);
      }
    }

    const oneHourAgo = moment().subtract(2, 'minutes').toISOString();

    // Fetch orders with awb_code, incomplete shipment status, and email not sent
    const pendingOrders = await Order.find({
      createdAt: { $lte: oneHourAgo },
      awb_code: { $exists: true },
      shipment_status: { $ne: "Delivered" },
      shippingemailSent: false
    });
    console.log(pendingOrders);

    if (pendingOrders.length === 0) {
      console.log("No pending orders to update.");
      return;
    }

    for (const order of pendingOrders) {
      const awb_number = order.awb_code;
      console.log("Processing order with AWB:", awb_number);

      const shipment_status_response = await axios.get(
        `https://apiv2.shiprocket.in/v1/external/courier/track/awb/${awb_number}`,
        {
          headers: {
            Authorization: `Bearer ${token}`
          }
        }
      );

      const trackingData = shipment_status_response.data.tracking_data.shipment_track;
      const latestStatus = trackingData[0]?.current_status || "Unknown";
       //const latestStatus='shipped';

      console.log(`Latest status for AWB ${awb_number}: ${latestStatus}`);
      order.shipment_status=latestStatus;
      await order.save();

      //checking the condition if the latestStatus is shipped then fire the product shipped email
      if (latestStatus === 'Shipped' || latestStatus === 'shipped') {
        console.log("Order shipped");
        const userName = order.user.name;
        console.log(userName);
        const userEmail = order.user.email;
        const shippingAddress1 = order.shippingAddress.address1;
        const zip_code=order.shippingAddress.zipCode;
        const city=order.shippingAddress.city;
        const state=order.shippingAddress.state
        const awb_code = order.awb_code;
        const productName = order.cart[0]?.name || "Product";

        console.log(`Sending email to ${userEmail} for order ${awb_code}`);
        console.log("the response going with the parameter is",productName, userEmail, userName, awb_code, shippingAddress1,zip_code,city,state);

        await productShippedEmail(productName, userEmail, userName, awb_code, shippingAddress1,zip_code,city,state);

        // Mark email as sent for shipping email
        order.shippingemailSent = true;
        //update the shipment_status  with the latest status fetch from the shiprocket api
        order.shipment_status = latestStatus;
        await order.save();
        console.log(`Email sent and order updated for ${awb_code}`);
      }

    }
  } catch (error) {
    console.log("Error in cron job:", error.message || error);
  }
});

//write another cron to generate the email of product is out of delivery
cron.schedule('*/30 * * * *', async () => {
  console.log("Running the crun to generate the email of product is out for delivery");
  try {
    const token = await generateShipRocketToken();

    async function productOutforDelivery(productName, BuyerEmail, Buyername,  awbnumber, delivery_address,city,state,zip_code) {
      var defaultClient = SibApiV3Sdk.ApiClient.instance;
      var apiKey = defaultClient.authentications['api-key'];
      apiKey.apiKey = process.env.MAILAPIKEY;

      var apiInstance = new SibApiV3Sdk.TransactionalEmailsApi();
      var adminMail = new SibApiV3Sdk.SendSmtpEmail();

      try {
        adminMail = {
          to: [
            { email: BuyerEmail },
            { email: 'gyanaranjan@ajatus.co.in' }
          ],
          from: {
            email: 'support@ceibagreen.com'
          },
          templateId: 35,
          params: {
            product_name: productName,
            Buyername: Buyername,
            awbnumber: awbnumber,
            delivery_address: delivery_address,
            city: city,
            state: state,
            zip_code: zip_code
          },
          subject: `Your product ${productName} is out for delivery`,
        };

        const data = await apiInstance.sendTransacEmail(adminMail);
        console.log("Mail sent successfully:", data.messageId);
      } catch (error) {
        console.log("Error sending email:", error);
      }
    }

    const oneHourAgo = moment().subtract(2, 'minutes').toISOString();

    // Fetch orders with awb_code, incomplete shipment status, and email not sent
    const pendingOrders = await Order.find({
      createdAt: { $lte: oneHourAgo },
      awb_code: { $exists: true },
      shipment_status: { $ne: "Delivered" },
      outfordeliverymailSent: false
    });

    if (pendingOrders.length === 0) {
      console.log("No pending orders for out of delivery.");
      return;
    }

    for (const order of pendingOrders) {
      const awb_number = order.awb_code;
      console.log("Processing order with AWB:", awb_number);

      const shipment_status_response = await axios.get(
        `https://apiv2.shiprocket.in/v1/external/courier/track/awb/${awb_number}`,
        {
          headers: {
            Authorization: `Bearer ${token}`
          }
        }
      );

      const trackingData = shipment_status_response.data.tracking_data.shipment_track;
      const latestStatus = trackingData[0]?.current_status || "Unknown";
       //const latestStatus='Out for Delivery';

      console.log(`Latest status for AWB ${awb_number}: ${latestStatus}`);

      //checking the condition if the latestStatus is shipped then fire the product shipped email
      if (latestStatus === 'Out for Delivery') {
        console.log("Order is out for delivery");
        const userName = order.user.name;
        const userEmail = order.user.email;
        const shippingAddress1 = order.shippingAddress.address1;
        const awb_code = order.awb_code;
        const zip_code=order.shippingAddress.zip_code;
        const city=order.shippingAddress.city;
        const state=order.shippingAddress.state
        const productName = order.cart[0]?.name || "Product";
        const deliverydate='Today';

        console.log(`Sending email to ${userEmail} for order ${awb_code}`);

        await productOutforDelivery(productName, userEmail, userName, deliverydate, awb_code, shippingAddress1,zip_code,city,state);

        // Mark email as sent for shipping email
        order.outfordeliverymailSent = true;
        //update the shipment_status  with the latest status fetch from the shiprocket api
        order.shipment_status = latestStatus;
        await order.save();
        console.log(`Email sent and order updated for ${awb_code}`);
      }

    }
  } catch (error) {
    console.log("Error in cron job:", error.message || error);
  }
});

//this crun will run for sent the email that your product is delivered successfully
cron.schedule('*/15 * * * *', async () => {
  console.log("Running the crun to send email the product which are already deliversd");
  try {
    const token = await generateShipRocketToken();

    async function productDelivered(productName, BuyerEmail, Buyername, awbnumber, delivery_address,orderId,zipcode,productImage,state,city,orderdate,mobile) {
      var defaultClient = SibApiV3Sdk.ApiClient.instance;
      var apiKey = defaultClient.authentications['api-key'];
      apiKey.apiKey = process.env.MAILAPIKEY;

      var apiInstance = new SibApiV3Sdk.TransactionalEmailsApi();
      var adminMail = new SibApiV3Sdk.SendSmtpEmail();
      console.log("Prodivy O,ahes",productImage)

      try {
        adminMail = {
          to: [
            { email: BuyerEmail },
            { email: 'gyanaranjan@ajatus.co.in' }
          ],
          from: {
            email: 'support@ceibagreen.com'
          },
          templateId: 29,
          params: {
            productName: productName,
            buyername: Buyername,
            // deliverydate: deliverydate,
            awbnumber: awbnumber,
            address: delivery_address,
            productImage: productImage,
            state: state,
            city: city,
            orderdate: orderdate,
            orderId:orderId,
            zipcode:zipcode,
            mobile:mobile
          },
          subject: `Product Delivered..`,
        };

        const data = await apiInstance.sendTransacEmail(adminMail);
        console.log("Mail sent successfully:", data.messageId);
      } catch (error) {
        console.log("Error sending email:", error);
      }
    }

    const oneHourAgo = moment().subtract(2, 'minutes').toISOString();

    // Fetch orders with awb_code, incomplete shipment status, and email not sent
    const pendingOrders = await Order.find({
      createdAt: { $lte: oneHourAgo },
      awb_code: { $exists: true },
      shipment_status: { $regex: /^delivered$/i },
      deliveredmailSent: false
    });
    //const productId=pendingOrders.cart[0].productId;
    

    if (pendingOrders.length === 0) {
      console.log("No pending orders to sent mail that the order is delivered.");
      return;
    }
    console.log("the pending order to sent email the product is deliverd is ",pendingOrders);

    for (const order of pendingOrders) {
      const awb_number = order.awb_code;
      console.log("Processing order with AWB:", awb_number);

      const shipment_status_response = await axios.get(
        `https://apiv2.shiprocket.in/v1/external/courier/track/awb/${awb_number}`,
        {
          headers: {
            Authorization: `Bearer ${token}`
          }
        }
      );

      const trackingData = shipment_status_response.data.tracking_data.shipment_track;
      const latestStatus = trackingData[0]?.current_status || "Unknown";
       //const latestStatus='delivered';

      console.log(`Latest status for AWB ${awb_number}: ${latestStatus}`);

      //checking the condition if the latestStatus is shipped then fire the product shipped email
      if (latestStatus === 'Delivered'||latestStatus==='delivered') {
        console.log("Order is delivered to the user successfully");
        const Buyername = order.user.name;
        const BuyerEmail = order.user.email;
        const delivery_address = order.shippingAddress.address1;
        const awb_code = order.awb_code;
        const productName = order.cart[0]?.name || "Product";
        const productId=order.cart[0].productId;
        console.log("the product id is",productId);
        const productDetails=await product.findById({_id:productId}).distinct('images');
        console.log("the product details fetched by the id is",productDetails);
        const deliverydate='Today';
        const orderId=order._id.toString();
        const zipcode=order.shippingAddress.zipCode;
        const state=order.shippingAddress.state;
        const city=order.shippingAddress.city;
        const country=order.shippingAddress.country;
        const productImagepath=productDetails[0];
        const productImage=`https://devzingaapi.ceibagreen.com/coverPhoto-${productImagepath}`;
        console.log("the productImage is",productImage);
        const awbnumber=order.awb_code;
        const orderdate=order.createdAt.toLocaleDateString();
        console.log("the order date ")
        const mobile=order.user.phone;


        console.log(`Sending email to ${BuyerEmail} for order ${awb_code}`);

        await productDelivered(productName, BuyerEmail, Buyername, awbnumber, delivery_address,orderId,zipcode,productImage,state,city,orderdate,mobile);

        // Mark email as sent for shipping email
        order.deliveredmailSent = true;
        //update the shipment_status  with the latest status fetch from the shiprocket api
        order.shipment_status = latestStatus;
        await order.save();
        console.log(`Email sent for order delivered to the user and order updated for ${awb_code}`);
      }

    }
  } catch (error) {
    console.log("Error in cron job:", error.message || error);
  }
});

  
  






// it'for errhendel
app.use(ErrorHandler);

// Handling Uncaught Exceptions
process.on("uncaughtException", (err) => {
  console.log(`Error: ${err.message}`);
  console.log(`shutting down the server for handling UNCAUGHT EXCEPTION! 💥`);
});

// unhandled promise rejection
process.on("unhandledRejection", (err) => {
  console.log(`Shutting down the server for ${err.message}`);
  console.log(`shutting down the server for unhandle promise rejection`);

  server.close(() => {
    process.exit(1);
  });
});
